@@ -34,8 +34,9 @@ pub mod iterators;
 #[cfg(not(feature = "std"))]
 use std::vec::Vec;
 
-use crate::core_iterators::{IntoIter, Iter};
+use crate::core_iterators::*;
 use crate::store::{Index, Position, Store};
+use crate::TryReserveError;
 use iterators::*;
 
 use std::borrow::Borrow;
@@ -88,21 +89,13 @@ use std::mem::replace;
 /// ```
 #[derive(Clone)]
 #[cfg(feature = "std")]
-pub struct DoublePriorityQueue<I, P, H = RandomState>
-where
-    I: Hash + Eq,
-    P: Ord,
-{
+pub struct DoublePriorityQueue<I, P, H = RandomState> {
     pub(crate) store: Store<I, P, H>,
 }
 
 #[derive(Clone)]
 #[cfg(not(feature = "std"))]
-pub struct DoublePriorityQueue<I, P, H>
-where
-    I: Hash + Eq,
-    P: Ord,
-{
+pub struct DoublePriorityQueue<I, P, H> {
     pub(crate) store: Store<I, P, H>,
 }
 
@@ -181,6 +174,17 @@ where
             store: Store::with_capacity_and_hasher(capacity, hash_builder),
         }
     }
+}
+
+impl<I, P, H> DoublePriorityQueue<I, P, H> {
+    /// Returns the number of elements the internal map can hold without
+    /// reallocating.
+    ///
+    /// This number is a lower bound; the map might be able to hold more,
+    /// but is guaranteed to be able to hold at least this many.
+    pub fn capacity(&self) -> usize {
+        self.store.capacity()
+    }
 
     /// Returns an iterator in arbitrary order over the
     /// (item, priority) elements in the queue
@@ -188,17 +192,88 @@ where
         self.store.iter()
     }
 
+    /// Clears the PriorityQueue, returning an iterator over the removed elements in arbitrary order.
+    /// If the iterator is dropped before being fully consumed, it drops the remaining elements in arbitrary order.
+    pub fn drain(&mut self) -> Drain<I, P> {
+        self.store.drain()
+    }
+
     /// Shrinks the capacity of the internal data structures
     /// that support this operation as much as possible.
     pub fn shrink_to_fit(&mut self) {
         self.store.shrink_to_fit();
     }
-}
 
+    /// Returns the number of elements in the priority queue.
+    #[inline]
+    pub fn len(&self) -> usize {
+        self.store.len()
+    }
+
+    /// Returns true if the priority queue contains no elements.
+    pub fn is_empty(&self) -> bool {
+        self.store.is_empty()
+    }
+
+    /// Returns the couple (item, priority) with the lowest
+    /// priority in the queue, or None if it is empty.
+    ///
+    /// Computes in **O(1)** time
+    pub fn peek_min(&self) -> Option<(&I, &P)> {
+        self.find_min().and_then(|i| {
+            self.store
+                .map
+                .get_index(unsafe { *self.store.heap.get_unchecked(i.0) }.0)
+        })
+    }
+
+    /// Reserves capacity for at least `additional` more elements to be inserted
+    /// in the given `DoublePriorityQueue`. The collection may reserve more space to avoid
+    /// frequent reallocations. After calling `reserve`, capacity will be
+    /// greater than or equal to `self.len() + additional`. Does nothing if
+    /// capacity is already sufficient.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the new capacity overflows `usize`.
+    pub fn reserve(&mut self, additional: usize) {
+        self.store.reserve(additional);
+    }
+
+    /// Reserve capacity for `additional` more elements, without over-allocating.
+    ///
+    /// Unlike `reserve`, this does not deliberately over-allocate the entry capacity to avoid
+    /// frequent re-allocations. However, the underlying data structures may still have internal
+    /// capacity requirements, and the allocator itself may give more space than requested, so this
+    /// cannot be relied upon to be precisely minimal.
+    ///
+    /// Computes in **O(n)** time.
+    pub fn reserve_exact(&mut self, additional: usize) {
+        self.store.reserve_exact(additional);
+    }
+
+    /// Try to reserve capacity for at least `additional` more elements.
+    ///
+    /// Computes in O(n) time.
+    pub fn try_reserve(&mut self, additional: usize) -> Result<(), TryReserveError> {
+        self.store.try_reserve(additional)
+    }
+
+    /// Try to reserve capacity for `additional` more elements, without over-allocating.
+    ///
+    /// Unlike `reserve`, this does not deliberately over-allocate the entry capacity to avoid
+    /// frequent re-allocations. However, the underlying data structures may still have internal
+    /// capacity requirements, and the allocator itself may give more space than requested, so this
+    /// cannot be relied upon to be precisely minimal.
+    ///
+    /// Computes in **O(n)** time.
+    pub fn try_reserve_exact(&mut self, additional: usize) -> Result<(), TryReserveError> {
+        self.store.try_reserve_exact(additional)
+    }
+}
 impl<I, P, H> DoublePriorityQueue<I, P, H>
 where
     P: Ord,
-    I: Hash + Eq,
 {
     /// Return an iterator in arbitrary order over the
     /// (item, priority) elements in the queue.
@@ -214,18 +289,6 @@ where
         IterMut::new(self)
     }
 
-    /// Returns the couple (item, priority) with the lowest
-    /// priority in the queue, or None if it is empty.
-    ///
-    /// Computes in **O(1)** time
-    pub fn peek_min(&self) -> Option<(&I, &P)> {
-        self.find_min().and_then(|i| {
-            self.store
-                .map
-                .get_index(unsafe { *self.store.heap.get_unchecked(i.0) }.0)
-        })
-    }
-
     /// Returns the couple (item, priority) with the greatest
     /// priority in the queue, or None if it is empty.
     ///
@@ -238,15 +301,6 @@ where
         })
     }
 
-    /// Returns the number of elements the internal map can hold without
-    /// reallocating.
-    ///
-    /// This number is a lower bound; the map might be able to hold more,
-    /// but is guaranteed to be able to hold at least this many.
-    pub fn capacity(&self) -> usize {
-        self.store.capacity()
-    }
-
     /// Removes the item with the lowest priority from
     /// the priority queue and returns the pair (item, priority),
     /// or None if the queue is empty.
@@ -295,17 +349,6 @@ where
         res
     }
 
-    /// Returns the number of elements in the priority queue.
-    #[inline]
-    pub fn len(&self) -> usize {
-        self.store.len()
-    }
-
-    /// Returns true if the priority queue contains no elements.
-    pub fn is_empty(&self) -> bool {
-        self.store.is_empty()
-    }
-
     /// Generates a new double ended iterator from self that
     /// will extract the elements from the one with the lowest priority
     /// to the highest one.
@@ -314,32 +357,237 @@ where
     }
 }
 
+impl<I, P, H> DoublePriorityQueue<I, P, H>
+where
+    H: BuildHasher,
+{
+    /// Returns the couple (item, priority) with the lowest
+    /// priority in the queue, or None if it is empty.
+    ///
+    /// The item is a mutable reference, but it's a logic error to modify it
+    /// in a way that change the result of  `Hash` or `Eq`.
+    ///
+    /// The priority cannot be modified with a call to this function.
+    /// To modify the priority use [`push`](DoublePriorityQueue::push),
+    /// [`change_priority`](DoublePriorityQueue::change_priority) or
+    /// [`change_priority_by`](DoublePriorityQueue::change_priority_by).
+    ///
+    /// Computes in **O(1)** time
+    pub fn peek_min_mut(&mut self) -> Option<(&mut I, &P)> {
+        use indexmap::map::MutableKeys;
+
+        self.find_min()
+            .and_then(move |i| {
+                self.store
+                    .map
+                    .get_index_mut2(unsafe { *self.store.heap.get_unchecked(i.0) }.0)
+            })
+            .map(|(k, v)| (k, &*v))
+    }
+}
+
+impl<I, P, H> DoublePriorityQueue<I, P, H>
+where
+    P: Ord,
+    H: BuildHasher,
+{
+    /// Returns the couple (item, priority) with the greatest
+    /// priority in the queue, or None if it is empty.
+    ///
+    /// The item is a mutable reference, but it's a logic error to modify it
+    /// in a way that change the result of  `Hash` or `Eq`.
+    ///
+    /// The priority cannot be modified with a call to this function.
+    /// To modify the priority use [`push`](DoublePriorityQueue::push),
+    /// [`change_priority`](DoublePriorityQueue::change_priority) or
+    /// [`change_priority_by`](DoublePriorityQueue::change_priority_by).
+    ///
+    /// Computes in **O(1)** time
+    pub fn peek_max_mut(&mut self) -> Option<(&mut I, &P)> {
+        use indexmap::map::MutableKeys;
+        self.find_max()
+            .and_then(move |i| {
+                self.store
+                    .map
+                    .get_index_mut2(unsafe { *self.store.heap.get_unchecked(i.0) }.0)
+            })
+            .map(|(k, v)| (k, &*v))
+    }
+}
+
 impl<I, P, H> DoublePriorityQueue<I, P, H>
 where
     P: Ord,
     I: Hash + Eq,
     H: BuildHasher,
 {
-    // reserve_exact -> IndexMap does not implement reserve_exact
+    /// Retains only the elements specified by the `predicate`.
+    ///
+    /// In other words, remove all elements e for which `predicate(&i, &p)` returns `false`.
+    /// The elements are visited in arbitrary order.
+    pub fn retain<F>(&mut self, predicate: F)
+    where
+        F: FnMut(&I, &P) -> bool,
+    {
+        self.store.retain(predicate);
+        self.heap_build();
+    }
 
-    /// Reserves capacity for at least `additional` more elements to be inserted
-    /// in the given `DoublePriorityQueue`. The collection may reserve more space to avoid
-    /// frequent reallocations. After calling `reserve`, capacity will be
-    /// greater than or equal to `self.len() + additional`. Does nothing if
-    /// capacity is already sufficient.
+    /// Retains only the elements specified by the `predicate`.
     ///
-    /// # Panics
+    /// In other words, remove all elements e for which `predicate(&mut i, &mut p)` returns `false`.
+    /// The elements are visited in arbitrary order.
     ///
-    /// Panics if the new capacity overflows `usize`.
-    pub fn reserve(&mut self, additional: usize) {
-        self.store.reserve(additional);
+    /// The `predicate` receives mutable references to both the item and
+    /// the priority.
+    ///
+    /// It's a logical error to change the item in a way
+    /// that changes the result of `Hash` or `Eq`.
+    ///
+    /// The `predicate` can change the priority. If the element is retained,
+    /// it will have the updated one.
+    pub fn retain_mut<F>(&mut self, predicate: F)
+    where
+        F: FnMut(&mut I, &mut P) -> bool,
+    {
+        self.store.retain_mut(predicate);
+        self.heap_build();
+    }
+
+    /// Returns an `Iterator` removing from the queue the `(item, priority)`
+    /// pairs for which the `predicate` returns `true`, in arbitraty order.
+    ///
+    /// The `predicate` receives mutable references to both the item and
+    /// the priority.
+    ///
+    /// It's a logical error to change the item in a way
+    /// that changes the result of `Hash` or `Eq`.
+    ///
+    /// The `predicate` can change the priority. If it returns `true`, the
+    /// extracted pair will have the updated priority, otherwise, the
+    /// heap structural property will be restored once the iterator is `Drop`ped.
+    ///
+    /// # Example
+    /// ```
+    /// # use priority_queue::DoublePriorityQueue;
+    /// let mut pq = DoublePriorityQueue::new();
+    ///
+    /// pq.push("Apples", 5);
+    /// pq.push("Bananas", 10);
+    ///
+    /// assert_eq!(pq.extract_if(|i, p| {
+    ///   *p = 15;
+    ///   i == &"Apples"
+    /// }).collect::<Vec<_>>(), vec![("Apples", 15)]);
+    ///
+    /// assert_eq!(pq.peek_min(), Some((&"Bananas", &15)));
+    /// assert_eq!(pq.into_vec(), vec!["Bananas"]);
+    /// ```
+    pub fn extract_if<F>(&mut self, predicate: F) -> ExtractIf<I, P, F, H>
+    where
+        F: FnMut(&mut I, &mut P) -> bool,
+    {
+        ExtractIf::new(self, predicate)
+    }
+
+    /// Removes the item with the lowest priority from
+    /// the priority queue if the predicate returns `true`.
+    ///
+    /// Returns the pair (item, priority), or None if the
+    /// queue is empty or the predicate returns `false`.
+    ///
+    /// The predicate receives mutable references to both the item and
+    /// the priority.
+    ///
+    /// It's a logical error to change the item in a way
+    /// that changes the result of `Hash` or `EQ`.
+    ///
+    /// The predicate can change the priority. If it returns true, the
+    /// returned couple will have the updated priority, otherwise, the
+    /// heap structural property will be restored.
+    ///
+    /// # Example
+    /// ```
+    /// # use priority_queue::DoublePriorityQueue;
+    /// let mut pq = DoublePriorityQueue::new();
+    ///
+    /// pq.push("Apples", 5);
+    /// pq.push("Bananas", 10);
+    ///
+    /// assert_eq!(pq.pop_min_if(|i, p| {
+    ///   *p = 15;
+    ///   false
+    /// }), None);
+    ///
+    /// assert_eq!(pq.pop_min(), Some(("Bananas", 10)));
+    /// ```
+    pub fn pop_min_if<F>(&mut self, f: F) -> Option<(I, P)>
+    where
+        F: FnOnce(&mut I, &mut P) -> bool,
+    {
+        self.find_min().and_then(|i| {
+            let r = self.store.swap_remove_if(i, f);
+            self.heapify(i);
+            r
+        })
+    }
+
+    /// Removes the item with the greatest priority from
+    /// the priority queue if the predicate returns `true`.
+    ///
+    /// Returns the pair (item, priority), or None if the
+    /// queue is empty or the predicate returns `false`.
+    ///
+    /// The predicate receives mutable references to both the item and
+    /// the priority.
+    ///
+    /// It's a logical error to change the item in a way
+    /// that changes the result of `Hash` or `EQ`.
+    ///
+    /// The predicate can change the priority. If it returns true, the
+    /// returned couple will have the updated priority, otherwise, the
+    /// heap structural property will be restored.
+    ///
+    /// # Example
+    /// ```
+    /// # use priority_queue::DoublePriorityQueue;
+    /// let mut pq = DoublePriorityQueue::new();
+    /// pq.push("Apples", 5);
+    /// pq.push("Bananas", 10);
+    /// assert_eq!(pq.pop_max_if(|i, p| {
+    ///   *p = 3;
+    ///   false
+    /// }), None);
+    /// assert_eq!(pq.pop_max(), Some(("Apples", 5)));
+    /// ```
+    pub fn pop_max_if<F>(&mut self, f: F) -> Option<(I, P)>
+    where
+        F: FnOnce(&mut I, &mut P) -> bool,
+    {
+        self.find_max().and_then(|i| {
+            let r = self.store.swap_remove_if(i, f);
+            self.up_heapify(i);
+            r
+        })
     }
 
     /// Insert the item-priority pair into the queue.
     ///
-    /// If an element equal to `item` was already into the queue,
-    /// it is updated and the old value of its priority is returned in `Some`;
-    /// otherwise, returns `None`.
+    /// If an element equal to `item` is already in the queue, its priority
+    /// is updated and the old priority is returned in `Some`; otherwise,
+    /// `item` is inserted with `priority` and `None` is returned.
+    ///
+    /// # Example
+    /// ```
+    /// # use priority_queue::DoublePriorityQueue;
+    /// let mut pq = DoublePriorityQueue::new();
+    /// assert_eq!(pq.push("Apples", 5), None);
+    /// assert_eq!(pq.get_priority("Apples"), Some(&5));
+    /// assert_eq!(pq.push("Apples", 6), Some(5));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&6));
+    /// assert_eq!(pq.push("Apples", 4), Some(6));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&4));
+    /// ```
     ///
     /// Computes in **O(log(N))** time.
     pub fn push(&mut self, item: I, priority: P) -> Option<P> {
@@ -377,13 +625,29 @@ where
     ///
     /// If an element equal to `item` is already in the queue with a
     /// lower priority, its priority is increased to the new one
-    /// without replacing the element and the old priority is returned.
-    /// Otherwise, the new element is inserted into the queue.
+    /// without replacing the element and the old priority is returned
+    /// in `Some`.
     ///
-    /// Returns `Some` if an element equal to `item` is already in the
-    /// queue. If its priority is higher then `priority`, the latter is returned back,
-    /// otherwise, the old priority is contained in the Option.
-    /// If the item is not in the queue, `None` is returned.
+    /// If an element equal to `item` is already in the queue with an
+    /// equal or higher priority, its priority is not changed and the
+    /// `priority` argument is returned in `Some`.
+    ///
+    /// If no element equal to `item` is already in the queue, the new
+    /// element is inserted and `None` is returned.
+    ///
+    /// # Example
+    /// ```
+    /// # use priority_queue::DoublePriorityQueue;
+    /// let mut pq = DoublePriorityQueue::new();
+    /// assert_eq!(pq.push_increase("Apples", 5), None);
+    /// assert_eq!(pq.get_priority("Apples"), Some(&5));
+    /// assert_eq!(pq.push_increase("Apples", 6), Some(5));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&6));
+    /// // Already present with higher priority, so requested (lower)
+    /// // priority is returned.
+    /// assert_eq!(pq.push_increase("Apples", 4), Some(4));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&6));
+    /// ```
     ///
     /// Computes in **O(log(N))** time.
     pub fn push_increase(&mut self, item: I, priority: P) -> Option<P> {
@@ -399,13 +663,29 @@ where
     ///
     /// If an element equal to `item` is already in the queue with a
     /// higher priority, its priority is decreased to the new one
-    /// without replacing the element and the old priority is returned.
-    /// Otherwise, the new element is inserted into the queue.
+    /// without replacing the element and the old priority is returned
+    /// in `Some`.
     ///
-    /// Returns `Some` if an element equal to `item` is already in the
-    /// queue. If its priority is lower then `priority`, the latter is returned back,
-    /// otherwise, the old priority is contained in the Option.
-    /// If the item is not in the queue, `None` is returned.
+    /// If an element equal to `item` is already in the queue with an
+    /// equal or lower priority, its priority is not changed and the
+    /// `priority` argument is returned in `Some`.
+    ///
+    /// If no element equal to `item` is already in the queue, the new
+    /// element is inserted and `None` is returned.
+    ///
+    /// # Example
+    /// ```
+    /// # use priority_queue::DoublePriorityQueue;
+    /// let mut pq = DoublePriorityQueue::new();
+    /// assert_eq!(pq.push_decrease("Apples", 5), None);
+    /// assert_eq!(pq.get_priority("Apples"), Some(&5));
+    /// assert_eq!(pq.push_decrease("Apples", 4), Some(5));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&4));
+    /// // Already present with lower priority, so requested (higher)
+    /// // priority is returned.
+    /// assert_eq!(pq.push_decrease("Apples", 6), Some(6));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&4));
+    /// ```
     ///
     /// Computes in **O(log(N))** time.
     pub fn push_decrease(&mut self, item: I, priority: P) -> Option<P> {
@@ -422,12 +702,24 @@ where
     /// The argument `item` is only used for lookup, and is not used to overwrite the item's data
     /// in the priority queue.
     ///
+    /// # Example
+    /// ```
+    /// # use priority_queue::DoublePriorityQueue;
+    /// let mut pq = DoublePriorityQueue::new();
+    /// assert_eq!(pq.change_priority("Apples", 5), None);
+    /// assert_eq!(pq.get_priority("Apples"), None);
+    /// assert_eq!(pq.push("Apples", 6), None);
+    /// assert_eq!(pq.get_priority("Apples"), Some(&6));
+    /// assert_eq!(pq.change_priority("Apples", 4), Some(6));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&4));
+    /// ```
+    ///
     /// The item is found in **O(1)** thanks to the hash table.
     /// The operation is performed in **O(log(N))** time.
-    pub fn change_priority<Q: ?Sized>(&mut self, item: &Q, new_priority: P) -> Option<P>
+    pub fn change_priority<Q>(&mut self, item: &Q, new_priority: P) -> Option<P>
     where
         I: Borrow<Q>,
-        Q: Eq + Hash,
+        Q: Eq + Hash + ?Sized,
     {
         self.store
             .change_priority(item, new_priority)
@@ -437,51 +729,6 @@ where
             })
     }
 
-    /// Returns the couple (item, priority) with the lowest
-    /// priority in the queue, or None if it is empty.
-    ///
-    /// The item is a mutable reference, but it's a logic error to modify it
-    /// in a way that change the result of  `Hash` or `Eq`.
-    ///
-    /// The priority cannot be modified with a call to this function.
-    /// To modify the priority use `push`, `change_priority` or
-    /// `change_priority_by`.
-    ///
-    /// Computes in **O(1)** time
-    pub fn peek_min_mut(&mut self) -> Option<(&mut I, &P)> {
-        use indexmap::map::MutableKeys;
-
-        self.find_min()
-            .and_then(move |i| {
-                self.store
-                    .map
-                    .get_index_mut2(unsafe { *self.store.heap.get_unchecked(i.0) }.0)
-            })
-            .map(|(k, v)| (k, &*v))
-    }
-
-    /// Returns the couple (item, priority) with the greatest
-    /// priority in the queue, or None if it is empty.
-    ///
-    /// The item is a mutable reference, but it's a logic error to modify it
-    /// in a way that change the result of  `Hash` or `Eq`.
-    ///
-    /// The priority cannot be modified with a call to this function.
-    /// To modify the priority use `push`, `change_priority` or
-    /// `change_priority_by`.
-    ///
-    /// Computes in **O(1)** time
-    pub fn peek_max_mut(&mut self) -> Option<(&mut I, &P)> {
-        use indexmap::map::MutableKeys;
-        self.find_max()
-            .and_then(move |i| {
-                self.store
-                    .map
-                    .get_index_mut2(unsafe { *self.store.heap.get_unchecked(i.0) }.0)
-            })
-            .map(|(k, v)| (k, &*v))
-    }
-
     /// Change the priority of an Item using the provided function.
     /// Return a boolean value where `true` means the item was in the queue and update was successful
     ///
@@ -490,10 +737,10 @@ where
     ///
     /// The item is found in **O(1)** thanks to the hash table.
     /// The operation is performed in **O(log(N))** time (worst case).
-    pub fn change_priority_by<Q: ?Sized, F>(&mut self, item: &Q, priority_setter: F) -> bool
+    pub fn change_priority_by<Q, F>(&mut self, item: &Q, priority_setter: F) -> bool
     where
         I: Borrow<Q>,
-        Q: Eq + Hash,
+        Q: Eq + Hash + ?Sized,
         F: FnOnce(&mut P),
     {
         self.store
@@ -505,20 +752,31 @@ where
     }
 
     /// Get the priority of an item, or `None`, if the item is not in the queue
-    pub fn get_priority<Q: ?Sized>(&self, item: &Q) -> Option<&P>
+    pub fn get_priority<Q>(&self, item: &Q) -> Option<&P>
     where
         I: Borrow<Q>,
-        Q: Eq + Hash,
+        Q: Eq + Hash + ?Sized,
     {
         self.store.get_priority(item)
     }
 
-    /// Get the couple (item, priority) of an arbitrary element, as reference
-    /// or `None` if the item is not in the queue.
-    pub fn get<Q: ?Sized>(&self, item: &Q) -> Option<(&I, &P)>
+    /// Check if the queue contains `item`.
+    ///
+    /// Returns `true` if `item` is in the queue, `false` if it is not.
+    pub fn contains<Q>(&self, item: &Q) -> bool
     where
         I: Borrow<Q>,
-        Q: Eq + Hash,
+        Q: Eq + Hash + ?Sized,
+    {
+        self.store.contains(item)
+    }
+
+    /// Get the couple (item, priority) of an arbitrary element, as reference
+    /// or `None` if the item is not in the queue.
+    pub fn get<Q>(&self, item: &Q) -> Option<(&I, &P)>
+    where
+        I: Borrow<Q>,
+        Q: Eq + Hash + ?Sized,
     {
         self.store.get(item)
     }
@@ -530,12 +788,13 @@ where
     /// in a way that change the result of  `Hash` or `Eq`.
     ///
     /// The priority cannot be modified with a call to this function.
-    /// To modify the priority use `push`, `change_priority` or
-    /// `change_priority_by`.
-    pub fn get_mut<Q: ?Sized>(&mut self, item: &Q) -> Option<(&mut I, &P)>
+    /// To modify the priority use  use [`push`](DoublePriorityQueue::push),
+    /// [`change_priority`](DoublePriorityQueue::change_priority) or
+    /// [`change_priority_by`](DoublePriorityQueue::change_priority_by).
+    pub fn get_mut<Q>(&mut self, item: &Q) -> Option<(&mut I, &P)>
     where
         I: Borrow<Q>,
-        Q: Eq + Hash,
+        Q: Eq + Hash + ?Sized,
     {
         self.store.get_mut(item)
     }
@@ -545,10 +804,10 @@ where
     /// is not found in the queue.
     ///
     /// The operation is performed in **O(log(N))** time (worst case).
-    pub fn remove<Q: ?Sized>(&mut self, item: &Q) -> Option<(I, P)>
+    pub fn remove<Q>(&mut self, item: &Q) -> Option<(I, P)>
     where
         I: Borrow<Q>,
-        Q: Eq + Hash,
+        Q: Eq + Hash + ?Sized,
     {
         self.store.remove(item).map(|(item, priority, pos)| {
             if pos.0 < self.len() {
@@ -574,25 +833,27 @@ where
     /// At the end, `other` will be empty.
     ///
     /// **Note** that at the end, the priority of the duplicated elements
-    /// inside self may be the one of the elements in other,
-    /// if other is longer than self
+    /// inside `self` may be the one of the elements in `other`,
+    /// if `other` is longer than `self`
     pub fn append(&mut self, other: &mut Self) {
         self.store.append(&mut other.store);
         self.heap_build();
     }
 }
 
-impl<I, P, H> DoublePriorityQueue<I, P, H>
-where
-    P: Ord,
-    I: Hash + Eq,
-{
+impl<I, P, H> DoublePriorityQueue<I, P, H> {
+    /// Returns the index of the min element
+    fn find_min(&self) -> Option<Position> {
+        match self.len() {
+            0 => None,
+            _ => Some(Position(0)),
+        }
+    }
 }
 
 impl<I, P, H> DoublePriorityQueue<I, P, H>
 where
     P: Ord,
-    I: Hash + Eq,
 {
     /**************************************************************************/
     /*                            internal functions                          */
@@ -770,12 +1031,13 @@ where
     }
 
     fn up_heapify(&mut self, i: Position) {
-        let tmp = unsafe { *self.store.heap.get_unchecked(i.0) };
-        let pos = self.bubble_up(i, tmp);
-        if i != pos {
-            self.heapify(i)
+        if let Some(&tmp) = self.store.heap.get(i.0) {
+            let pos = self.bubble_up(i, tmp);
+            if i != pos {
+                self.heapify(i)
+            }
+            self.heapify(pos);
         }
-        self.heapify(pos);
     }
 
     /// Internal function that transform the `heap`
@@ -805,14 +1067,6 @@ where
             ),
         }
     }
-
-    /// Returns the index of the min element
-    fn find_min(&self) -> Option<Position> {
-        match self.len() {
-            0 => None,
-            _ => Some(Position(0)),
-        }
-    }
 }
 
 //FIXME: fails when the vector contains repeated items
