@@ -1,4 +1,6 @@
-use proc_macro2::TokenStream;
+use core::fmt::Display;
+
+use proc_macro2::{Span, TokenStream};
 use quote::{format_ident, quote};
 use syn::{
     parse_quote, spanned::Spanned as _, DataEnum, Error, Field, Fields,
@@ -42,6 +44,10 @@ pub fn impl_enum(
         public.extend(generate_archived_type(
             printing, attributes, generics, data,
         )?);
+
+        private.extend(generate_niching_impls(
+            printing, attributes, generics, data,
+        )?);
     }
 
     public.extend(generate_resolver_type(
@@ -194,7 +200,7 @@ fn generate_archived_type(
         let variant_doc = variant_doc(name, variant_name);
 
         let mut variant_fields = TokenStream::new();
-        for field in variant.fields.iter() {
+        for (i, field) in variant.fields.iter().enumerate() {
             let Field {
                 vis,
                 ident,
@@ -203,9 +209,20 @@ fn generate_archived_type(
             } = field;
             let field_attrs = FieldAttributes::parse(attributes, field)?;
 
+            let field_doc = format!(
+                "The archived counterpart of [`{}::{}::{}`]",
+                name,
+                variant_name,
+                ident.as_ref().map_or_else(
+                    || &i as &dyn Display,
+                    |name| name as &dyn Display
+                )
+            );
+
             let field_ty = field_attrs.archived(rkyv_path, field);
             let field_metas = field_attrs.metas();
             variant_fields.extend(quote! {
+                #[doc = #field_doc]
                 #field_metas
                 #vis #ident #colon_token #field_ty,
             });
@@ -348,8 +365,8 @@ fn generate_resolve_arms(
             .enumerate()
             .map(|(i, field)| {
                 (
-                    Ident::new(&format!("self_{}", i), field.span()),
-                    Ident::new(&format!("resolver_{}", i), field.span()),
+                    Ident::new(&format!("self_{i}"), field.span()),
+                    Ident::new(&format!("resolver_{i}"), field.span()),
                 )
             })
             .unzip::<_, _, Vec<_>, Vec<_>>();
@@ -563,8 +580,8 @@ fn generate_partial_eq_impl(
             .enumerate()
             .map(|(i, f)| {
                 (
-                    Ident::new(&format!("self_{}", i), f.span()),
-                    Ident::new(&format!("other_{}", i), f.span()),
+                    Ident::new(&format!("self_{i}"), f.span()),
+                    Ident::new(&format!("other_{i}"), f.span()),
                 )
             })
             .unzip::<_, _, Vec<_>, Vec<_>>();
@@ -692,8 +709,8 @@ fn generate_partial_ord_impl(
             .enumerate()
             .map(|(i, f)| {
                 (
-                    Ident::new(&format!("self_{}", i), f.span()),
-                    Ident::new(&format!("other_{}", i), f.span()),
+                    Ident::new(&format!("self_{i}"), f.span()),
+                    Ident::new(&format!("other_{i}"), f.span()),
                 )
             })
             .unzip::<_, _, Vec<_>, Vec<_>>();
@@ -796,3 +813,148 @@ fn generate_partial_ord_impl(
         }
     })
 }
+
+fn generate_niching_impls(
+    printing: &Printing,
+    attributes: &Attributes,
+    generics: &Generics,
+    data: &DataEnum,
+) -> Result<TokenStream, Error> {
+    let Printing {
+        rkyv_path,
+        archived_type,
+        ..
+    } = printing;
+
+    let (impl_generics, ty_generics, ..) = generics.split_for_impl();
+
+    let mut result = TokenStream::new();
+
+    let mut niches = Vec::new();
+
+    for variant in data.variants.iter() {
+        let variant_name = &variant.ident;
+        let archived_variant_name =
+            format_ident!("ArchivedVariant{}", strip_raw(variant_name));
+
+        for (i, field) in variant.fields.iter().enumerate() {
+            let field_attrs = FieldAttributes::parse(attributes, field)?;
+            let archived_field = field_attrs.archived(rkyv_path, field);
+
+            for niche in field_attrs.niches {
+                let niche_tokens = niche.to_tokens(rkyv_path);
+
+                // Best-effort attempt at improving the error message if the
+                // same `Niching` implementor type is being used multiple times.
+                // Otherwise, the compiler will inform about conflicting impls
+                // which are not entirely unreasonable but may appear slightly
+                // cryptic.
+                if niches.contains(&niche) {
+                    return Err(Error::new_spanned(
+                        niche_tokens,
+                        "each niching type may be used at most once",
+                    ));
+                }
+
+                let field_member = if let Some(ref name) = field.ident {
+                    Member::Named(name.clone())
+                } else {
+                    Member::Unnamed(Index::from(i + 1))
+                };
+
+                let tag_member = if field.ident.is_some() {
+                    Member::Named(Ident::new("__tag", Span::call_site()))
+                } else {
+                    Member::Unnamed(Index::from(0))
+                };
+
+                let field_niche = quote! {
+                    <#niche_tokens as #rkyv_path::niche::niching::Niching<
+                        #archived_field>
+                    >
+                };
+
+                result.extend(quote! {
+                    #[automatically_derived]
+                    impl #impl_generics
+                        #rkyv_path::niche::niching::Niching<#archived_type>
+                    for #niche_tokens {
+                        unsafe fn is_niched(niched: *const #archived_type)
+                            -> bool
+                        {
+                            let variant = niched
+                                .cast::<#archived_variant_name #ty_generics>();
+                            let tag = unsafe {
+                                ::core::ptr::addr_of!((*variant).#tag_member)
+                            };
+                            if unsafe { &*tag != &ArchivedTag::#variant_name } {
+                                return false;
+                            }
+                            let field = unsafe {
+                                ::core::ptr::addr_of!((*variant).#field_member)
+                            };
+                            unsafe { #field_niche::is_niched(field) }
+                        }
+
+                        fn resolve_niched(
+                            out: #rkyv_path::Place<#archived_type>
+                        ) {
+                            let out = unsafe {
+                                out.cast_unchecked::<
+                                    #archived_variant_name #ty_generics
+                                >()
+                            };
+                            let tag_ptr = unsafe {
+                                ::core::ptr::addr_of_mut!(
+                                    (*out.ptr()).#tag_member
+                                )
+                            };
+                            unsafe {
+                                tag_ptr.write(ArchivedTag::#variant_name);
+                            }
+                            let field_ptr = unsafe {
+                                ::core::ptr::addr_of_mut!(
+                                    (*out.ptr()).#field_member
+                                )
+                            };
+                            let out_field = unsafe {
+                                #rkyv_path::Place::from_field_unchecked(
+                                    out, field_ptr,
+                                )
+                            };
+                            #field_niche::resolve_niched(out_field);
+                        }
+                    }
+                });
+
+                niches.push(niche);
+            }
+        }
+    }
+
+    let mut iter = niches.iter();
+
+    while let Some(niche1) = iter.next() {
+        let niche1_tokens = niche1.to_tokens(rkyv_path);
+        for niche2 in iter.clone() {
+            let niche2_tokens = niche2.to_tokens(rkyv_path);
+            result.extend(quote! {
+                #[automatically_derived]
+                unsafe impl #impl_generics
+                    #rkyv_path::niche::niching::SharedNiching<
+                        #niche1_tokens, #niche2_tokens
+                    >
+                for #archived_type {}
+
+                #[automatically_derived]
+                unsafe impl #impl_generics
+                    #rkyv_path::niche::niching::SharedNiching<
+                        #niche2_tokens, #niche1_tokens
+                    >
+                for #archived_type {}
+            });
+        }
+    }
+
+    Ok(result)
+}
